@@ -2,6 +2,7 @@ import logging
 from datetime import datetime, timedelta
 from django.conf import settings
 from django.shortcuts import render
+from django.utils.translation import gettext_lazy as _
 from django.contrib import messages
 from django.views.generic import FormView, TemplateView
 from django.urls import reverse_lazy, reverse
@@ -106,7 +107,7 @@ class Twilio2FAMixin(object):
         elif not len(self.allowed_methods):
             messages.error(
                 request,
-                "No verification method is available."
+                _("No verification method is available")
             )
             return self.get_error_redirect(
                 can_retry=False
@@ -137,7 +138,7 @@ class Twilio2FAMixin(object):
             # Verification not found
             messages.error(
                 self.request,
-                "The verification has expired. Please try again."
+                _("The verification has expired. Please try again.")
             )
             return self.get_redirect("start")
 
@@ -155,7 +156,8 @@ class Twilio2FAMixin(object):
     def clear_session(self):
         for key in self.SESSION_VALUES:
             key_ = f"{SESSION_PREFIX}_{key}"
-            if key not in self.request.session:
+
+            if key_ not in self.request.session:
                 continue
 
             del self.request.session[key_]
@@ -181,23 +183,28 @@ class Twilio2FAVerificationMixin(Twilio2FAMixin):
         except ValidationError:
             self.phone_number = None
 
+        self.timeout_value = get_setting(
+            "TIMEOUT_CB",
+            callback_kwargs={
+                "user": request.user
+            }
+        )
+
     def dispatch(self, request, *args, **kwargs):
-        timeout_value = self.get_session_value(SESSION_TIMEOUT)
-
-        if timeout_value:
-            timeout_value = datetime.strptime(timeout_value, DATEFMT)
-
-            if timeout_value > datetime.now(tz=timeout_value.tzinfo):
+        if self.timeout_value:
+            if self.timeout_value > datetime.now(tz=self.timeout_value.tzinfo):
                 messages.error(
                     request,
-                    "You cannot make another 2FA verification attempt at this time."
+                    _("You cannot make another verification attempt at this time.")
                 )
                 return self.get_error_redirect(can_retry=False)
+            else:
+                self.clear_session()
 
         if not self.phone_number:
             messages.warning(
                 request,
-                "You must add a phone number to your account before proceeding."
+                _("You must add a phone number to your account before proceeding.")
             )
             return self.get_redirect("register")
 
@@ -265,7 +272,7 @@ class Twilio2FAVerificationMixin(Twilio2FAMixin):
             return True
 
         twilio_2fa_verification_status_changed.send(
-            None,
+            sender=None,
             user=self.request.user,
             phone_number=self.e164_phone_number(),
             method=self.get_session_value(SESSION_METHOD),
@@ -379,7 +386,7 @@ class Twilio2FAChangeView(Twilio2FARegistrationFormView):
         if not ctx["can_change"]:
             messages.error(
                 self.request,
-                "You are not allowed to make changes to your phone number."
+                _("You are not allowed to make changes to your phone number.")
             )
 
         return ctx
@@ -436,7 +443,7 @@ class Twilio2FAStartView(Twilio2FAVerificationMixin, TemplateView):
         if elapsed.total_seconds() < min_retry_wait:
             messages.warning(
                 request,
-                "Please allow up to 3 minutes before retrying."
+                _(f"Please allow at least {round(min_retry_wait / 60, 0)} minutes before retrying.")
             )
             return self.get_redirect("verify")
 
@@ -453,7 +460,7 @@ class Twilio2FAStartView(Twilio2FAVerificationMixin, TemplateView):
 
         messages.success(
             request,
-            "Verification has been re-sent."
+            _("Verification has been re-sent.")
         )
 
         return self.get_redirect("verify")
@@ -464,7 +471,7 @@ class Twilio2FAStartView(Twilio2FAVerificationMixin, TemplateView):
         if method not in self.allowed_methods:
             messages.error(
                 request,
-                "The form has been tampered with. Please don't do that."
+                _("The form has been tampered with. Please don't do that.")
             )
             return self.get(request, *args, **kwargs)
 
@@ -519,8 +526,8 @@ class Twilio2FAStartView(Twilio2FAVerificationMixin, TemplateView):
             if e.code == 60223:
                 messages.error(
                     self.request,
-                    f"Unable to verify using {self.AVAILABLE_METHODS[method]['label']} at this time. "
-                    f"Please try a different method."
+                    _(f"Unable to verify using {self.AVAILABLE_METHODS[method]['label']} at this time. "
+                      f"Please try a different method.")
                 )
                 return self.get_redirect("start")
 
@@ -542,24 +549,29 @@ class Twilio2FAVerifyView(Twilio2FAVerificationMixin, FormView):
     def handle_too_many_attempts(self):
         self.cancel_verification()
 
-        timeout = get_setting(
+        timeout_seconds = get_setting(
             "MAX_ATTEMPTS_TIMEOUT",
             default=600
         )
 
-        if timeout:
-            self.set_session_value(
-                SESSION_TIMEOUT,
-                datetime.strftime(datetime.now() + timedelta(seconds=int(timeout)), DATEFMT)
+        if timeout_seconds:
+            twilio_2fa_verification_retries_exceeded.send(
+                sender=None,
+                user=self.request.user,
+                phone_number=self.phone_number,
+                method=self.get_session_value(SESSION_METHOD),
+                twilio_sid=self.get_session_value(SESSION_SID),
+                timeout=timeout_seconds,
+                timeout_until=datetime.now() + timedelta(seconds=int(timeout_seconds))
             )
 
         messages.error(
             self.request,
-            "You have made too many attempts to verify."
+            _("You have made too many attempts to verify.")
         )
 
         return self.get_error_redirect(
-            can_retry=True if not timeout else False
+            can_retry=True if not timeout_seconds else False
         )
 
     def form_valid(self, form):
@@ -598,7 +610,7 @@ class Twilio2FAVerifyView(Twilio2FAVerificationMixin, FormView):
         if verify.status == "approved":
             # Send this signal manually
             twilio_2fa_verification_status_changed.send(
-                None,
+                sender=None,
                 user=self.request.user,
                 phone_number=self.e164_phone_number(),
                 method=self.get_session_value(SESSION_METHOD),
@@ -607,7 +619,7 @@ class Twilio2FAVerifyView(Twilio2FAVerificationMixin, FormView):
             )
 
             twilio_2fa_verification_success.send(
-                None,
+                sender=None,
                 user=self.request.user,
                 phone_number=self.e164_phone_number(),
                 method=self.get_session_value(SESSION_METHOD)
@@ -617,7 +629,7 @@ class Twilio2FAVerifyView(Twilio2FAVerificationMixin, FormView):
 
         messages.error(
             self.request,
-            "Verification code was invalid"
+            _("Verification code was invalid")
         )
 
         return super().form_invalid(form)
